@@ -22,8 +22,8 @@ from sphinx.util import format_exception_cut_frames, save_traceback
 from sphinx.application import Sphinx
 from sphinx.util.console import darkred, nocolor
 
-__revision__ = '$Revision: 61950 $'
-__version__ = '0.1.' + __revision__[11:-2]
+__revision__ = '$Revision: 62438 $'
+__version__ = '0.2'
 
 
 def usage(argv, msg=None):
@@ -147,7 +147,7 @@ def main(argv=sys.argv):
                 print >>sys.stderr, ('Please also report this if it was a user '
                                      'error, so that a better error message '
                                      'can be provided next time.')
-                print >>sys.stderr, 'Send reports to georg@python.org. Thanks!'
+                print >>sys.stderr, 'Send reports to sphinx-dev@googlegroups.com. Thanks!'
             return 1
 
 
