@@ -18,10 +18,11 @@ from docutils import nodes
 from docutils.parsers.rst import directives, roles
 
 import sphinx
-from sphinx.roles import xfileref_role
+from sphinx.roles import xfileref_role, innernodetypes
 from sphinx.config import Config
 from sphinx.builder import builtin_builders
-from sphinx.directives import desc_directive, additional_xref_types
+from sphinx.directives import desc_directive, target_directive, additional_xref_types
+from sphinx.environment import SphinxStandaloneReader
 from sphinx.util.console import bold
 
 
@@ -79,6 +80,9 @@ class Sphinx(object):
         # load all extension modules
         for extension in getattr(self.config, 'extensions', ()):
             self.setup_extension(extension)
+        # the config file itself can be an extension
+        if hasattr(self.config, 'setup'):
+            self.config.setup(self)
 
         # this must happen after loading extension modules, since they
         # can add custom config values
@@ -195,8 +199,50 @@ class Sphinx(object):
     def add_role(self, name, role):
         roles.register_canonical_role(name, role)
 
-    def add_description_unit(self, directivename, rolename, indexdesc='',
-                             parse_node=None):
-        additional_xref_types[directivename] = (rolename, indexdesc, parse_node)
+    def add_description_unit(self, directivename, rolename, indextemplate='',
+                             parse_node=None, ref_nodeclass=None):
+        additional_xref_types[directivename] = (rolename, indextemplate, parse_node)
         directives.register_directive(directivename, desc_directive)
         roles.register_canonical_role(rolename, xfileref_role)
+        if ref_nodeclass is not None:
+            innernodetypes[rolename] = ref_nodeclass
+
+    def add_crossref_type(self, directivename, rolename, indextemplate='',
+                          ref_nodeclass=None):
+        additional_xref_types[directivename] = (rolename, indextemplate, None)
+        directives.register_directive(directivename, target_directive)
+        roles.register_canonical_role(rolename, xfileref_role)
+        if ref_nodeclass is not None:
+            innernodetypes[rolename] = ref_nodeclass
+
+    def add_transform(self, transform):
+        SphinxStandaloneReader.transforms.append(transform)
+
+
+class TemplateBridge(object):
+    """
+    
+    """
+
+    def init(self, builder):
+        """
+        Called by the builder to initialize the template system.  *builder*
+        is the builder object; you'll probably want to look at the value of
+        ``builder.config.templates_path``.
+        """
+        raise NotImplementedError('must be implemented in subclasses')
+
+    def newest_template_mtime(self):
+        """
+        Called by the builder to determine if output files are outdated
+        because of template changes.  Return the mtime of the newest template
+        file that was changed.  The default implementation returns ``0``.
+        """
+        return 0
+
+    def render(self, template, context):
+        """
+        Called by the builder to render a *template* with a specified
+        context (a Python dictionary).
+        """
+        raise NotImplementedError('must be implemented in subclasses')
