@@ -74,27 +74,14 @@ class Builder(object):
         raise NotImplementedError
 
     def init_templates(self):
-        """Call if you need Jinja templates in the builder."""
-        # lazily import this, other builders won't need it
-        from sphinx._jinja import Environment, SphinxFileSystemLoader
-
-        # load templates
-        self.templates = {}
-        base_templates_path = path.join(path.dirname(__file__), 'templates')
-        ext_templates_path = [path.join(self.srcdir, dir)
-                              for dir in self.config.templates_path]
-        self.templates_path = [base_templates_path] + ext_templates_path
-        loader = SphinxFileSystemLoader(base_templates_path, ext_templates_path)
-        self.jinja_env = Environment(loader=loader,
-                                     # disable traceback, more likely that something
-                                     # in the application is broken than in the templates
-                                     friendly_traceback=False)
-
-    def get_template(self, name):
-        if name in self.templates:
-            return self.templates[name]
-        template = self.templates[name] = self.jinja_env.get_template(name)
-        return template
+        # Call this from init() if you need templates.
+        if self.config.template_bridge:
+            self.templates = self.app.import_object(
+                self.config.template_bridge, 'template_bridge setting')()
+        else:
+            from sphinx._jinja import BuiltinTemplates
+            self.templates = BuiltinTemplates()
+        self.templates.init(self)
 
     def get_target_uri(self, docname, typ=None):
         """
@@ -330,17 +317,26 @@ class StandaloneHTMLBuilder(Builder):
         else:
             self.last_updated = None
 
+        docstitle = self.config.html_title or \
+                    '%s v%s documentation' % (self.config.project,
+                                              self.config.release)
+        logo = self.config.html_logo and \
+               path.basename(self.config.html_logo) or ''
+
         self.globalcontext = dict(
             project = self.config.project,
-            copyright = self.config.copyright,
             release = self.config.release,
             version = self.config.version,
             last_updated = self.last_updated,
+            copyright = self.config.copyright,
             style = self.config.html_style,
             use_modindex = self.config.html_use_modindex,
+            use_opensearch = self.config.html_use_opensearch,
+            docstitle = docstitle,
             builder = self.name,
             parents = [],
             titles = {},
+            logo = logo,
             len = len, # the built-in
         )
 
@@ -476,11 +472,10 @@ class StandaloneHTMLBuilder(Builder):
             self.info(' '+pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
-        # the index page
-        indextemplate = self.config.html_index
-        if indextemplate:
-            self.info(' index', nonl=1)
-            self.handle_page('index', {'indextemplate': indextemplate}, 'index.html')
+        if self.config.html_use_opensearch:
+            self.info(' opensearch', nonl=1)
+            fn = path.join(self.outdir, '_static', 'opensearch.xml')
+            self.handle_page('opensearch', {}, 'opensearch.xml', outfilename=fn)
 
         self.info()
 
@@ -492,6 +487,12 @@ class StandaloneHTMLBuilder(Builder):
                 self.info(' '+src, nonl=1)
                 shutil.copyfile(path.join(self.srcdir, src),
                                 path.join(self.outdir, '_images', dest))
+            # the logo is handled differently
+            if self.config.html_logo:
+                logobase = path.basename(self.config.html_logo)
+                self.info(' '+logobase, nonl=1)
+                shutil.copyfile(path.join(self.srcdir, self.config.html_logo),
+                                path.join(self.outdir, '_static', logobase))
             self.info()
 
         # copy static files
@@ -514,8 +515,8 @@ class StandaloneHTMLBuilder(Builder):
         self.handle_finish()
 
     def get_outdated_docs(self):
-        if self.templates_path:
-            template_mtime = max(mtimes_of_files(self.templates_path, '.html'))
+        if self.templates:
+            template_mtime = self.templates.newest_template_mtime()
         else:
             template_mtime = 0
         for docname in self.env.found_docs:
@@ -527,9 +528,14 @@ class StandaloneHTMLBuilder(Builder):
                 targetmtime = path.getmtime(targetname)
             except Exception:
                 targetmtime = 0
-            srcmtime = max(path.getmtime(self.env.doc2path(docname)), template_mtime)
-            if srcmtime > targetmtime:
-                yield docname
+            try:
+                srcmtime = max(path.getmtime(self.env.doc2path(docname)),
+                               template_mtime)
+                if srcmtime > targetmtime:
+                    yield docname
+            except EnvironmentError:
+                # source doesn't exist anymore
+                pass
 
     def load_indexer(self, docnames):
         try:
@@ -553,7 +559,8 @@ class StandaloneHTMLBuilder(Builder):
     def get_target_uri(self, docname, typ=None):
         return docname + '.html'
 
-    def handle_page(self, pagename, addctx, templatename='page.html'):
+    def handle_page(self, pagename, addctx, templatename='page.html',
+                    outfilename=None):
         ctx = self.globalcontext.copy()
         ctx['current_page_name'] = pagename
 
@@ -569,8 +576,9 @@ class StandaloneHTMLBuilder(Builder):
             ctx['customsidebar'] = sidebarfile
         ctx.update(addctx)
 
-        output = self.get_template(templatename).render(ctx)
-        outfilename = path.join(self.outdir, os_path(pagename) + '.html')
+        output = self.templates.render(templatename, ctx)
+        if not outfilename:
+            outfilename = path.join(self.outdir, os_path(pagename) + '.html')
         ensuredir(path.dirname(outfilename)) # normally different from self.outdir
         try:
             f = codecs.open(outfilename, 'w', 'utf-8')
@@ -606,8 +614,7 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
 
     def init(self):
         self.init_translator_class()
-        # no templates used, but get_outdated_docs() needs this attribute
-        self.templates_path = []
+        self.templates = None   # no template bridge necessary
 
     def get_target_uri(self, docname, typ=None):
         if docname == 'index':
@@ -616,12 +623,14 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
             return docname[:-5] # up to sep
         return docname + SEP
 
-    def handle_page(self, pagename, ctx, templatename='page.html'):
+    def handle_page(self, pagename, ctx, templatename='page.html',
+                    outfilename=None):
         ctx['current_page_name'] = pagename
         sidebarfile = self.config.html_sidebars.get(pagename, '')
         if sidebarfile:
             ctx['customsidebar'] = path.join(self.srcdir, sidebarfile)
-        outfilename = path.join(self.outdir, os_path(pagename) + '.fpickle')
+        if not outfilename:
+            outfilename = path.join(self.outdir, os_path(pagename) + '.fpickle')
         ensuredir(path.dirname(outfilename))
         f = open(outfilename, 'wb')
         try:
@@ -806,6 +815,13 @@ class LaTeXBuilder(Builder):
                                 path.join(self.outdir, dest))
             self.info()
 
+        # the logo is handled differently
+        if self.config.latex_logo:
+            logobase = path.basename(self.config.latex_logo)
+            self.info(' '+logobase, nonl=1)
+            shutil.copyfile(path.join(self.srcdir, self.config.latex_logo),
+                            path.join(self.outdir, logobase))
+
         self.info(bold('copying TeX support files...'))
         staticdirname = path.join(path.dirname(__file__), 'texinputs')
         for filename in os.listdir(staticdirname):
@@ -822,9 +838,6 @@ class ChangesBuilder(Builder):
 
     def init(self):
         self.init_templates()
-        self.ftemplate = self.get_template('changes/frameset.html')
-        self.vtemplate = self.get_template('changes/versionchanges.html')
-        self.stemplate = self.get_template('changes/rstsource.html')
 
     def get_outdated_docs(self):
         return self.outdir
@@ -871,21 +884,26 @@ class ChangesBuilder(Builder):
                 otherchanges.setdefault((docname, title), []).append(
                     (entry, docname, lineno))
 
+        docstitle = self.config.html_title or \
+                    '%s v%s documentation' % (self.config.project,
+                                              self.config.release)
+
         ctx = {
             'project': self.config.project,
             'version': version,
+            'docstitle': docstitle,
             'libchanges': sorted(libchanges.iteritems()),
             'apichanges': sorted(apichanges),
             'otherchanges': sorted(otherchanges.iteritems()),
         }
         f = open(path.join(self.outdir, 'index.html'), 'w')
         try:
-            f.write(self.ftemplate.render(ctx))
+            f.write(self.templates.render('changes/frameset.html', ctx))
         finally:
             f.close()
         f = open(path.join(self.outdir, 'changes.html'), 'w')
         try:
-            f.write(self.vtemplate.render(ctx))
+            f.write(self.templates.render('changes/versionchanges.html', ctx))
         finally:
             f.close()
 
@@ -911,7 +929,7 @@ class ChangesBuilder(Builder):
             try:
                 text = ''.join(hl(i+1, line) for (i, line) in enumerate(lines))
                 ctx = {'filename': self.env.doc2path(docname, None), 'text': text}
-                f.write(self.stemplate.render(ctx))
+                f.write(self.templates.render('changes/rstsource.html', ctx))
             finally:
                 f.close()
         shutil.copyfile(path.join(path.dirname(__file__), 'static', 'default.css'),
