@@ -10,7 +10,6 @@
 """
 
 import os
-import types
 from os import path
 
 
@@ -43,18 +42,21 @@ class Config(object):
         add_module_names = (True, True),
         show_authors = (False, True),
         pygments_style = ('sphinx', False),
+        template_bridge = (None, False),
 
         # HTML options
+        html_title = (None, False),
         html_style = ('default.css', False),
+        html_logo = (None, False),
         html_static_path = ([], False),
         html_last_updated_fmt = ('%b %d, %Y', False),
         html_use_smartypants = (True, False),
         html_translator_class = (None, False),
-        html_index = ('', False),
         html_sidebars = ({}, False),
         html_additional_pages = ({}, False),
         html_use_modindex = (True, False),
         html_copy_source = (True, False),
+        html_use_opensearch = (False, False),
 
         # HTML help options
         htmlhelp_basename = ('pydoc', False),
@@ -63,6 +65,7 @@ class Config(object):
         latex_paper_size = ('letter', False),
         latex_font_size = ('10pt', False),
         latex_documents = ([], False),
+        latex_logo = (None, False),
         latex_preamble = ('', False),
         latex_appendices = ([], False),
         latex_use_modindex = (True, False),
@@ -77,10 +80,6 @@ class Config(object):
             execfile(config['__file__'], config)
         finally:
             os.chdir(olddir)
-        # remove potentially pickling-problematic values
-        for key, val in config.items():
-            if key.startswith('_') or isinstance(val, types.ModuleType):
-                del config[key]
         self.__dict__.update(config)
 
     def init_defaults(self):
@@ -91,5 +90,11 @@ class Config(object):
     def __getitem__(self, name):
         return getattr(self, name)
 
+    def __setitem__(self, name, value):
+        setattr(self, name, value)
+
+    def __delitem__(self, name):
+        delattr(self, name)
+
     def __contains__(self, name):
         return hasattr(self, name)
