@@ -19,8 +19,11 @@ from docutils import nodes
 from docutils.parsers.rst import directives
 
 from sphinx import addnodes
+from sphinx.roles import caption_ref_re
 from sphinx.util.compat import make_admonition
 
+ws_re = re.compile(r'\s+')
+
 # ------ index markup --------------------------------------------------------------
 
 entrytypes = [
@@ -137,12 +140,16 @@ def parse_py_signature(signode, sig, desctype, env):
     if env.currclass:
         if classname and classname.startswith(env.currclass):
             fullname = classname + name
+            # class name is given again in the signature
             classname = classname[len(env.currclass):].lstrip('.')
             add_module = False
         elif classname:
+            # class name is given in the signature, but different
             fullname = env.currclass + '.' + classname + name
         else:
+            # class name is not given in the signature
             fullname = env.currclass + '.' + name
+            add_module = False
     else:
         fullname = classname and classname + name or name
 
@@ -183,7 +190,7 @@ def parse_py_signature(signode, sig, desctype, env):
 
 c_sig_re = re.compile(
     r'''^([^(]*?)          # return type
-        (\w+)  \s*         # thing name
+        ([\w:]+)  \s*      # thing name (colon allowed for C++ class names)
         (?: \((.*)\) )? $  # optionally arguments
     ''', re.VERBOSE)
 c_funcptr_sig_re = re.compile(
@@ -213,7 +220,7 @@ def parse_c_type(node, ctype):
             node += tnode
 
 def parse_c_signature(signode, sig, desctype):
-    """Transform a C-language signature into RST nodes."""
+    """Transform a C (or C++) signature into RST nodes."""
     # first try the function pointer signature regex, it's more specific
     m = c_funcptr_sig_re.match(sig)
     if m is None:
@@ -269,16 +276,25 @@ def parse_opcode_signature(signode, sig):
     return opname.strip()
 
 
-option_desc_re = re.compile(r'([-/])([-_a-zA-Z0-9]+)(\s*.*)')
+option_desc_re = re.compile(
+    r'(/|-|--)([-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)')
 
 def parse_option_desc(signode, sig):
     """Transform an option description into RST nodes."""
-    m = option_desc_re.match(sig)
-    if m is None: raise ValueError
-    prefix, optname, args = m.groups()
-    signode += addnodes.desc_name(prefix+optname, prefix+optname)
-    signode += addnodes.desc_classname(args, args)
-    return optname
+    count = 0
+    firstname = ''
+    for m in option_desc_re.finditer(sig):
+        prefix, optname, args = m.groups()
+        if count:
+            signode += addnodes.desc_classname(', ', ', ')
+        signode += addnodes.desc_name(prefix+optname, prefix+optname)
+        signode += addnodes.desc_classname(args, args)
+        if not count:
+            firstname = optname
+        count += 1
+    if not firstname:
+        raise ValueError
+    return firstname
 
 
 def desc_directive(desctype, arguments, options, content, lineno,
@@ -324,20 +340,26 @@ def desc_directive(desctype, arguments, options, content, lineno,
                 continue
             else:
                 # another registered generic x-ref directive
-                rolename, indextext, parse_node = additional_xref_types[desctype]
+                rolename, indextemplate, parse_node = additional_xref_types[desctype]
                 if parse_node:
                     fullname = parse_node(env, sig, signode)
                 else:
                     signode.clear()
                     signode += addnodes.desc_name(sig, sig)
-                    fullname = sig
+                    # normalize whitespace like xfileref_role does
+                    fullname = ws_re.sub('', sig)
                 if not noindex:
                     targetname = '%s-%s' % (rolename, fullname)
                     signode['ids'].append(targetname)
                     state.document.note_explicit_target(signode)
-                    if indextext:
-                        env.note_index_entry('pair',
-                                             '%s; %s' % (indextext, fullname),
+                    if indextemplate:
+                        indexentry = indextemplate % (fullname,)
+                        indextype = 'single'
+                        colon = indexentry.find(':')
+                        if colon != -1:
+                            indextype = indexentry[:colon].strip()
+                            indexentry = indexentry[colon+1:].strip()
+                        env.note_index_entry(indextype, indexentry,
                                              targetname, targetname)
                     env.note_reftarget(rolename, fullname, targetname)
                 # don't use object indexing below
@@ -367,7 +389,7 @@ def desc_directive(desctype, arguments, options, content, lineno,
     subnode = addnodes.desc_content()
     # needed for automatic qualification of members
     clsname_set = False
-    if desctype == 'class' and names:
+    if desctype in ('class', 'exception') and names:
         env.currclass = names[0]
         clsname_set = True
     elif desctype in ('method', 'attribute') and clsname and not env.currclass:
@@ -413,13 +435,41 @@ desctypes = [
 for _name in desctypes:
     directives.register_directive(_name, desc_directive)
 
-# Generic cross-reference types; they can be registered in the application
+# Generic cross-reference types; they can be registered in the application;
+# the directives are either desc_directive or target_directive
 additional_xref_types = {
-    # directive name: (role name, index text)
-    'envvar': ('envvar', 'environment variable', None),
+    # directive name: (role name, index text, function to parse the desc node)
+    'envvar': ('envvar', 'environment variable; %s', None),
 }
 
 
+# ------ target --------------------------------------------------------------------
+
+def target_directive(targettype, arguments, options, content, lineno,
+                     content_offset, block_text, state, state_machine):
+    """Generic target for user-defined cross-reference types."""
+    env = state.document.settings.env
+    rolename, indextemplate, _ = additional_xref_types[targettype]
+    # normalize whitespace in fullname like xfileref_role does
+    fullname = ws_re.sub('', arguments[0].strip())
+    targetname = '%s-%s' % (rolename, fullname)
+    node = nodes.target('', '', ids=[targetname])
+    state.document.note_explicit_target(node)
+    if indextemplate:
+        indexentry = indextemplate % (fullname,)
+        indextype = 'single'
+        colon = indexentry.find(':')
+        if colon != -1:
+            indextype = indexentry[:colon].strip()
+            indexentry = indexentry[colon+1:].strip()
+        env.note_index_entry(indextype, indexentry, targetname, targetname)
+    env.note_reftarget(rolename, fullname, targetname)
+    return [node]
+
+target_directive.content = 0
+target_directive.arguments = (1, 0, 1)
+
+
 # ------ versionadded/versionchanged -----------------------------------------------
 
 def version_directive(name, arguments, options, content, lineno,
@@ -550,6 +600,19 @@ module_directive.options = {'platform': lambda x: x,
 directives.register_directive('module', module_directive)
 
 
+def currentmodule_directive(name, arguments, options, content, lineno,
+                            content_offset, block_text, state, state_machine):
+    # This directive is just to tell people that we're documenting
+    # stuff in module foo, but links to module foo won't lead here.
+    env = state.document.settings.env
+    modname = arguments[0].strip()
+    env.currmodule = modname
+    return []
+
+currentmodule_directive.arguments = (1, 0, 0)
+directives.register_directive('currentmodule', currentmodule_directive)
+
+
 def author_directive(name, arguments, options, content, lineno,
                      content_offset, block_text, state, state_machine):
     # Show authors only if the show_authors option is on
@@ -586,19 +649,26 @@ def toctree_directive(name, arguments, options, content, lineno,
     ret = []
     subnode = addnodes.toctree()
     includefiles = []
+    includetitles = {}
     for docname in content:
         if not docname:
             continue
+        # look for explicit titles and documents ("Some Title <document>").
+        m = caption_ref_re.match(docname)
+        if m:
+            docname = m.group(2)
+            includetitles[docname] = m.group(1)    
         # absolutize filenames, remove suffixes
         if docname.endswith(suffix):
             docname = docname[:-len(suffix)]
         docname = posixpath.normpath(posixpath.join(dirname, docname))
         if docname not in env.found_docs:
             ret.append(state.document.reporter.warning(
-                'toctree references unknown document %s' % docname, line=lineno))
+                'toctree references unknown document %r' % docname, line=lineno))
         else:
             includefiles.append(docname)
     subnode['includefiles'] = includefiles
+    subnode['includetitles'] = includetitles
     subnode['maxdepth'] = options.get('maxdepth', -1)
     ret.append(subnode)
     return ret
