@@ -13,6 +13,7 @@ import re
 import os
 import time
 import heapq
+import types
 import difflib
 import itertools
 import cPickle as pickle
@@ -60,18 +61,6 @@ default_settings = {
 ENV_VERSION = 21
 
 
-def walk_depth(node, depth, maxdepth):
-    """Utility: Cut a TOC at a specified depth."""
-    for subnode in node.children[:]:
-        if isinstance(subnode, (addnodes.compact_paragraph, nodes.list_item)):
-            walk_depth(subnode, depth, maxdepth)
-        elif isinstance(subnode, nodes.bullet_list):
-            if depth > maxdepth:
-                subnode.parent.replace(subnode, [])
-            else:
-                walk_depth(subnode, depth+1, maxdepth)
-
-
 default_substitutions = set([
     'version',
     'release',
@@ -142,17 +131,18 @@ class HandleCodeBlocks(Transform):
                 node.replace_self(node.children[0])
 
 
-class MyStandaloneReader(standalone.Reader):
+class SphinxStandaloneReader(standalone.Reader):
     """
     Add our own transforms.
     """
+    transforms = [DefaultSubstitutions, MoveModuleTargets,
+                  FilterMessages, HandleCodeBlocks]
     def get_transforms(self):
         tf = standalone.Reader.get_transforms(self)
-        return tf + [DefaultSubstitutions, MoveModuleTargets,
-                     FilterMessages, HandleCodeBlocks]
+        return tf + self.transforms
 
 
-class MyContentsFilter(ContentsFilter):
+class SphinxContentsFilter(ContentsFilter):
     """
     Used with BuildEnvironment.add_toc_from() to discard cross-file links
     within table-of-contents link nodes.
@@ -167,10 +157,6 @@ class BuildEnvironment:
     The environment in which the ReST files are translated.
     Stores an inventory of cross-file targets and provides doctree
     transformations to resolve links to them.
-
-    Not all doctrees are stored in the environment, only those of files
-    containing a "toctree" directive, because they have to change if sections
-    are edited in other files. This keeps the environment size moderate.
     """
 
     # --------- ENVIRONMENT PERSISTENCE ----------------------------------------
@@ -191,6 +177,13 @@ class BuildEnvironment:
         warnfunc = self._warnfunc
         self.set_warnfunc(None)
         picklefile = open(filename, 'wb')
+        # remove potentially pickling-problematic values from config
+        for key, val in vars(self.config).items():
+            if key.startswith('_') or \
+                   isinstance(val, types.ModuleType) or \
+                   isinstance(val, types.FunctionType) or \
+                   isinstance(val, (type, types.ClassType)):
+                del self.config[key]
         try:
             pickle.dump(self, picklefile, pickle.HIGHEST_PROTOCOL)
         finally:
@@ -433,11 +426,13 @@ class BuildEnvironment:
                 del self.images[imgsrc]
 
 
-    # --------- SINGLE FILE BUILDING -------------------------------------------
+    # --------- SINGLE FILE READING --------------------------------------------
 
     def read_doc(self, docname, src_path=None, save_parsed=True, app=None):
-        """Parse a file and add/update inventory entries for the doctree.
-        If srcpath is given, read from a different source file."""
+        """
+        Parse a file and add/update inventory entries for the doctree.
+        If srcpath is given, read from a different source file.
+        """
         # remove all inventory entries for that file
         self.clear_doc(docname)
 
@@ -447,7 +442,7 @@ class BuildEnvironment:
         self.docname = docname
         doctree = publish_doctree(None, src_path, FileInput,
                                   settings_overrides=self.settings,
-                                  reader=MyStandaloneReader())
+                                  reader=SphinxStandaloneReader())
         self.process_dependencies(docname, doctree)
         self.process_images(docname, doctree)
         self.process_metadata(docname, doctree)
@@ -558,7 +553,7 @@ class BuildEnvironment:
         """
         for node in document.traverse(nodes.section):
             titlenode = nodes.title()
-            visitor = MyContentsFilter(document)
+            visitor = SphinxContentsFilter(document)
             node[0].walkabout(visitor)
             titlenode += visitor.get_entry_text()
             self.titles[docname] = titlenode
@@ -624,7 +619,7 @@ class BuildEnvironment:
                 title = subnode[0]
                 # copy the contents of the section title, but without references
                 # and unnecessary stuff
-                visitor = MyContentsFilter(document)
+                visitor = SphinxContentsFilter(document)
                 title.walkabout(visitor)
                 nodetext = visitor.get_entry_text()
                 if not numentries[0]:
@@ -715,7 +710,7 @@ class BuildEnvironment:
         self.resolve_references(doctree, docname, builder)
 
         # now, resolve all toctree nodes
-        def _entries_from_toctree(toctreenode):
+        def _entries_from_toctree(toctreenode, separate=False):
             """Return TOC entries for a toctree node."""
             includefiles = map(str, toctreenode['includefiles'])
 
@@ -728,21 +723,47 @@ class BuildEnvironment:
                     self.warn(docname, 'toctree contains ref to nonexisting '
                               'file %r' % includefile)
                 else:
+                    # resolve all sub-toctrees
                     for toctreenode in toc.traverse(addnodes.toctree):
-                        toctreenode.parent.replace_self(
-                            _entries_from_toctree(toctreenode))
-                    entries.append(toc)
-            if entries:
-                return addnodes.compact_paragraph('', '', *entries)
-            return None
+                        i = toctreenode.parent.index(toctreenode) + 1
+                        for item in _entries_from_toctree(toctreenode):
+                            toctreenode.parent.insert(i, item)
+                            i += 1
+                        toctreenode.parent.remove(toctreenode)
+                    if separate:
+                        entries.append(toc)
+                    else:
+                        entries.extend(toc.children)
+            return entries
+
+        def _walk_depth(node, depth, maxdepth, titleoverrides):
+            """Utility: Cut a TOC at a specified depth."""
+            for subnode in node.children[:]:
+                if isinstance(subnode, (addnodes.compact_paragraph, nodes.list_item)):
+                    _walk_depth(subnode, depth, maxdepth, titleoverrides)
+                elif isinstance(subnode, nodes.bullet_list):
+                    if depth > maxdepth:
+                        subnode.parent.replace(subnode, [])
+                    else:
+                        _walk_depth(subnode, depth+1, maxdepth, titleoverrides)
 
         for toctreenode in doctree.traverse(addnodes.toctree):
             maxdepth = toctreenode.get('maxdepth', -1)
-            newnode = _entries_from_toctree(toctreenode)
-            if newnode is not None:
-                # prune the tree to maxdepth
+            titleoverrides = toctreenode.get('includetitles', {})
+            tocentries = _entries_from_toctree(toctreenode, separate=True)
+            if tocentries:
+                newnode = addnodes.compact_paragraph('', '', *tocentries)
+                # prune the tree to maxdepth and replace titles
                 if maxdepth > 0:
-                    walk_depth(newnode, 1, maxdepth)
+                    _walk_depth(newnode, 1, maxdepth, titleoverrides)
+                # replace titles, if needed
+                if titleoverrides:
+                    for refnode in newnode.traverse(nodes.reference):
+                        if refnode.get('anchorname', None):
+                            continue
+                        if refnode['refuri'] in titleoverrides:
+                            newtitle = titleoverrides[refnode['refuri']]
+                            refnode.children = [nodes.Text(newtitle)]
                 toctreenode.replace_self(newnode)
             else:
                 toctreenode.replace_self([])
