@@ -41,7 +41,8 @@ def prepare_docstring(s):
     """
     if not s or s.isspace():
         return ['']
-    nl = s.expandtabs().rstrip().find('\n')
+    s = s.expandtabs()
+    nl = s.rstrip().find('\n')
     if nl == -1:
         # Only one line...
         return [s.strip(), '']
@@ -70,7 +71,7 @@ def get_module_charset(module):
 
 
 def generate_rst(what, name, members, undoc, add_content, document, lineno,
-                 indent='', filename_set=None):
+                 indent='', filename_set=None, check_module=False):
     env = document.settings.env
 
     # find out what to import
@@ -99,6 +100,13 @@ def generate_rst(what, name, members, undoc, add_content, document, lineno,
 
     result = ViewList()
 
+    if mod is None:
+        warning = document.reporter.warning(
+            'don\'t know which module to import for documenting %r '
+            '(try placing a "module" or "currentmodule" directive in the document, '
+            'or giving an explicit module name)' % name, line=lineno)
+        return [warning], result
+
     try:
         todoc = module = __import__(mod, None, None, ['foo'])
         if filename_set is not None and hasattr(module, '__file__') and module.__file__:
@@ -108,9 +116,11 @@ def generate_rst(what, name, members, undoc, add_content, document, lineno,
             filename_set.add(modfile)
         for part in objpath:
             todoc = getattr(todoc, part)
-        if hasattr(todoc, '__module__'):
-            if todoc.__module__ != mod:
-                return [], result
+        if check_module:
+            # only checking __module__ for members not given explicitly
+            if hasattr(todoc, '__module__'):
+                if todoc.__module__ != mod:
+                    return [], result
         docstring = todoc.__doc__
     except (ImportError, AttributeError):
         warning = document.reporter.warning(
@@ -122,6 +132,10 @@ def generate_rst(what, name, members, undoc, add_content, document, lineno,
     try:
         if what == 'class':
             args = inspect.formatargspec(*inspect.getargspec(todoc.__init__))
+            if args[1:7] == 'self, ':
+                args = '(' + args[7:]
+            elif args == '(self)':
+                args = '()'
         elif what in ('function', 'method'):
             args = inspect.formatargspec(*inspect.getargspec(todoc))
             if what == 'method':
@@ -173,7 +187,12 @@ def generate_rst(what, name, members, undoc, add_content, document, lineno,
     warnings = []
     # add members, if possible
     _all = members == ['__all__']
+    members_check_module = False
     if _all:
+        if what == 'module':
+            # for implicit module members, check __module__ to avoid documenting
+            # imported objects
+            members_check_module = True
         all_members = sorted(inspect.getmembers(todoc))
     else:
         all_members = [(mname, getattr(todoc, mname)) for mname in members]
@@ -205,7 +224,8 @@ def generate_rst(what, name, members, undoc, add_content, document, lineno,
                 continue
         full_membername = name + '.' + membername
         subwarn, subres = generate_rst(memberwhat, full_membername, ['__all__'],
-                                       undoc, None, document, lineno, indent)
+                                       undoc, None, document, lineno, indent,
+                                       check_module=members_check_module)
         warnings.extend(subwarn)
         result.extend(subres)
 
