@@ -10,7 +10,7 @@
 """
 
 import sys
-from os import path
+import posixpath
 
 from docutils import nodes
 from docutils.writers.html4css1 import Writer, HTMLTranslator as BaseTranslator
@@ -252,8 +252,8 @@ class HTMLTranslator(BaseTranslator):
         olduri = node['uri']
         # rewrite the URI if the environment knows about it
         if olduri in self.builder.env.images:
-            node['uri'] = path.join(self.builder.imgpath,
-                                    self.builder.env.images[olduri][1])
+            node['uri'] = posixpath.join(self.builder.imgpath,
+                                         self.builder.env.images[olduri][1])
         BaseTranslator.visit_image(self, node)
 
     def visit_toctree(self, node):
@@ -351,8 +351,11 @@ class SmartyPantsHTMLTranslator(HTMLTranslator):
         finally:
             self.no_smarty -= 1
 
-    def encode(self, text):
-        text = HTMLTranslator.encode(self, text)
-        if self.no_smarty <= 0:
-            text = sphinx_smarty_pants(text)
-        return text
+    def visit_Text(self, node):
+        text = node.astext()
+        encoded = self.encode(text)
+        if self.in_mailto and self.settings.cloak_email_addresses:
+            encoded = self.cloak_email(encoded)
+        elif self.no_smarty <= 0:
+            encoded = sphinx_smarty_pants(encoded)
+        self.body.append(encoded)
